from django.core.management.base import BaseCommand, CommandError
from ohm2_handlers_light import utils as h_utils
from ohm2_backoffice_light import utils as ohm2_backoffice_light_utils
import os

class Command(BaseCommand):
	
	def add_arguments(self, parser):
		parser.add_argument('-u', '--username')
		parser.add_argument('-e', '--email')
		parser.add_argument('-p', '--password')

	def handle(self, *args, **options):
		username = options["username"]
		email = options["email"]
		password = options["password"]
		
		ohm2_backoffice_light_utils.create_staff_user(username, email, password)



from django import VERSION as DJANGO_VERSION

if DJANGO_VERSION >= (2, 0):
	from django.urls import include, re_path as url
else:
	from django.conf.urls import include, url

from . import views

app_name = "ohm2_backoffice_light"

urlpatterns = [
	url(r'^ohm2_backoffice_light/$', views.index, name = 'index'),		
]





from django.contrib.auth.decorators import login_required
from guardian.decorators import permission_required_or_403
from ohm2_handlers_light.decorators import ohm2_handlers_light_safe_request
from . import settings

def ohm2_backoffice_light_safe_request(function):
	return ohm2_handlers_light_safe_request(function, "ohm2_backoffice_light")

def ohm2_backoffice_light_staff_views_or_403(func):

	@login_required(login_url = settings.LOGIN_URL)
	@permission_required_or_403("ohm2_backoffice_light." + settings.STAFF_PERMISSION_ENTER_VIEW[0], raise_exception = True)
	def wrapper(*args, **kwargs):
		return func(*args, **kwargs)

	return wrapper


from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.utils.translation import ugettext as _
from ohm2_handlers_light.models import BaseModel
from . import managers
from . import settings



class Staff(BaseModel):
	user = models.OneToOneField(User, on_delete = models.CASCADE)

	class Meta:
		permissions = settings.STAFF_PERMISSIONS


from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User, Group
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from ohm2_accounts_light import utils as ohm2_accounts_light_utils
from guardian.models import UserObjectPermission
from guardian.shortcuts import assign_perm
from . import models as ohm2_backoffice_light_models
from . import errors as ohm2_backoffice_light_errors
from . import settings
import os, time, random


random_string = "bcD8tYixjLe0gE9jW7V3NzlTepaM9m6q"



def create_staff(user, **kwargs):
	kwargs["user"] = user
	return h_utils.db_create(ohm2_backoffice_light_models.Staff, **kwargs)

def get_or_none_staff(**kwargs):
	return h_utils.db_get_or_none(ohm2_backoffice_light_models.Staff, **kwargs)
	
def get_staff(**kwargs):
	return h_utils.db_get(ohm2_backoffice_light_models.Staff, **kwargs)

def filter_staff(**kwargs):
	return h_utils.db_filter(ohm2_backoffice_light_models.Staff, **kwargs)

def q_staff(q):
	return h_utils.db_q(ohm2_backoffice_light_models.Staff, q)	

def get_staff_group():
	group = get_or_none_staff_group()
	if group:
		return group
	return create_staff_group()	

def get_or_none_staff_group():
	return h_utils.db_get_or_none(Group, name = "staff")

def create_staff_group():
	staff_group = h_utils.db_create(Group, name = "staff")
	return staff_group

def assign_permissions_to_staff_group(group, **kwargs):
	for perm in settings.STAFF_PERMISSIONS:
		assign_perm("ohm2_backoffice_light." + perm[0], group)
	return group	

def create_staff_user(username, email, password, **kwargs):
	user = ohm2_accounts_light_utils.create_user(username, email, password)

	try:
		user = h_utils.db_update(user, is_staff = True)
		staff = create_staff(user)
		
		staff_group = get_staff_group()

		staff_group = assign_permissions_to_staff_group(staff_group)

		user.groups.add(staff_group)		

	except Exception as e:
		h_utils.db_delete(user)
		raise RunException(**ohm2_backoffice_light_errors.FAILED_TO_CREATE_STAFF_USER)
	
	return user



from django.shortcuts import render, redirect
from django.urls import reverse
from django.http import HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.decorators import authentication_classes, permission_classes, api_view
from ohm2_handlers_light.parsers import get_as_or_get_default
from ohm2_backoffice_light.decorators import ohm2_backoffice_light_staff_views_or_403
from . import dispatcher




@api_view(['POST'])
@authentication_classes((SessionAuthentication,))
@permission_classes((IsAuthenticated,))
@ohm2_backoffice_light_staff_views_or_403
def user_exist(request):
	"""
	Check if user exist

	__Inputs__:

		- username (string, optional)
		- email (string, optional)


	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if exist or false if doesn't exist

	__Notes__:

		- At least one input must be present

	"""
	keys = (
		("username", "username", ""),
		("email", "email", ""),
	)
	res, error = dispatcher.user_exist(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)



@api_view(['POST'])
@authentication_classes((SessionAuthentication,))
@permission_classes((IsAuthenticated,))
@ohm2_backoffice_light_staff_views_or_403
def is_password_secure(request):
	"""
	Check if user's password is secure

	__Inputs__:

		- password (string, required)
		- username (string, optional)


	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if secure or false if not secure

	__Notes__:

		- At least one input must be present

	"""
	keys = (
		("username", "username", ""),
		("password", "password", ""),
	)
	res, error = dispatcher.is_password_secure(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)




from ohm2_handlers_light import utils as h_utils
from ohm2_accounts_light import utils as ohm2_accounts_light_utils
from ohm2_backoffice_light.decorators import ohm2_backoffice_light_safe_request
from . import errors as ohm2_backoffice_light_api_v1_errors




@ohm2_backoffice_light_safe_request
def user_exist(request, params):
	p = h_utils.cleaned(params, (
							("string", "username", 0),
							("string", "email", 0),
						))

	
	username, email = p["username"], p["email"]
	
	if len(username) > 0:
		query = {"username" : username}

	elif len(email) > 0:
		query = {"email" : email}

	else:
		return {"error" : ohm2_backoffice_light_api_v1_errors.BOTH_CANT_BE_EMPTY}
	

	exist = ohm2_accounts_light_utils.user_exist(**query)
	

	res = {
		"error" : None,
		"ret" : exist,
	}
	return res



@ohm2_backoffice_light_safe_request
def is_password_secure(request, params):
	p = h_utils.cleaned(params, (
							("string", "username", 1),
							("string", "password", 1),
						))

	
	username, password = p["username"], p["password"]
	
	user = ohm2_accounts_light_utils.get_or_none_user(username = p["username"])
	secure = ohm2_accounts_light_utils.is_password_secure(password, user)
	
	#errors = ohm2_accounts_light_utils.validate_current_password(password, user)

	res = {
		"error" : None,
		"ret" : secure,
	}
	return res





from django import VERSION as DJANGO_VERSION

if DJANGO_VERSION >= (2, 0):
	from django.urls import include, re_path as url
else:
	from django.conf.urls import include, url


from . import views

app_name = "ohm2_backoffice_light_api_v1"

urlpatterns = [
	url(r'^user-exist/$', views.user_exist, name = 'user_exist'),
	url(r'^is-password-secure/$', views.is_password_secure, name = 'is_password_secure'),
]